@@ -92,4 +92,11 @@ fn main() {
         .page::<User>(1, 10).unwrap();
 
     // ...
+
+    // Transaction
+    entity_manager.start_transaction().and_then(|mut transaction| {
+        let list: Vec<User> = transaction.list(Wrapper::new().eq("name", "Jack"))?;
+        let insert_id: Option<i32> = transaction.save(&User::default())?;
+        transaction.commit()
+    }).unwrap();
 }
\ No newline at end of file
